@@ -7,8 +7,8 @@
 
 #ifndef ALPHA_TARGET_ELF_H
 #define ALPHA_TARGET_ELF_H
-static inline const char *cpu_get_model(uint32_t eflags)
-{
-    return "ev67";
-}
+
+#define ELF_CLASS               ELFCLASS64
+#define ELF_MACHINE             EM_ALPHA
+
 #endif
