@@ -4,15 +4,17 @@
 #
 # SPDX-License-Identifier: GPL-2.0-or-later
 
-import ansible_runner
+import ansible_runner  # type: ignore
 import logging
 import shutil
 import yaml
 
+from ansible_runner import Runner
 from pathlib import Path
 from tempfile import TemporaryDirectory, NamedTemporaryFile
 
 from lcitool import util, LcitoolError
+from typing import Any, Dict, List, Optional, Union
 
 log = logging.getLogger(__name__)
 
@@ -24,14 +26,14 @@ class AnsibleWrapperError(LcitoolError):
     types.
     """
 
-    def __init__(self, message):
+    def __init__(self, message: str):
         super().__init__(message, "AnsibleWrapper")
 
 
 class ExecutionError(AnsibleWrapperError):
     """Thrown whenever the Ansible runner failed the execution."""
 
-    def __init__(self, message):
+    def __init__(self, message: str):
         message_prefix = "Ansible execution failed: "
         message = message_prefix + message
         super().__init__(message)
@@ -40,19 +42,20 @@ class ExecutionError(AnsibleWrapperError):
 class EnvironmentError(AnsibleWrapperError):
     """Thrown when preparation of the execution environment failed."""
 
-    def __init__(self, message):
+    def __init__(self, message: str):
         message_prefix = "Failed to prepare the execution environment: "
         message = message_prefix + message
         super().__init__(message)
 
 
-class AnsibleWrapper():
-    def __init__(self):
-        self._tempdir = TemporaryDirectory(prefix="ansible_runner",
-                                           dir=util.get_temp_dir())
-        self._private_data_dir = Path(self._tempdir.name)
+class AnsibleWrapper:
+    def __init__(self) -> None:
+        self._tempdir: TemporaryDirectory = TemporaryDirectory(
+            prefix="ansible_runner", dir=util.get_temp_dir()
+        )
+        self._private_data_dir: Path = Path(self._tempdir.name)
 
-    def _get_default_params(self):
+    def _get_default_params(self) -> Dict[str, Union[str, Any]]:
         ansible_log_path = Path(util.get_cache_dir(), "ansible.log").as_posix()
         default_params = {
             "private_data_dir": self._private_data_dir,
@@ -62,20 +65,29 @@ class AnsibleWrapper():
                 "ANSIBLE_NOCOWS": "True",
                 "ANSIBLE_LOG_PATH": ansible_log_path,
                 "ANSIBLE_SSH_PIPELINING": "True",
-
                 # Group names officially cannot contain dashes, because those
                 # characters are invalid in Python identifiers and it caused
                 # issues in some Ansible scenarios like using the dot notation,
                 # e.g. groups.group-with-dash. In simple group names like
                 # ours dashes are still perfectly fine, so ignore the warning
                 "ANSIBLE_TRANSFORM_INVALID_GROUP_CHARS": "ignore",
-            }
+            },
         }
 
         return default_params
 
-    def prepare_env(self, playbookdir=None, inventories=None,
-                    group_vars=None, extravars=None):
+    def prepare_env(
+        self,
+        playbookdir: Optional[Path] = None,
+        inventories: Optional[List[Union[Path, Dict[str, Any]]]] = None,
+        group_vars: Optional[
+            Dict[
+                str,
+                Any,
+            ]
+        ] = None,
+        extravars: Optional[Dict[str, Any]] = None,
+    ) -> None:
         """
         Prepares the Ansible runner execution environment.
 
@@ -120,10 +132,10 @@ class AnsibleWrapper():
             # all the sources, otherwise we'd lose data due to rewriting the
             # impacted files.
             for inventory in inventories:
-                if type(inventory) is dict:
+                if isinstance(inventory, dict):
                     with NamedTemporaryFile("w", dir=dst, delete=False) as fd:
                         yaml.dump(inventory, fd)
-                else:
+                elif isinstance(inventory, Path) or isinstance(inventory, str):
                     if inventory.is_dir():
                         shutil.copytree(inventory, dst, dirs_exist_ok=True)
                     else:
@@ -134,8 +146,7 @@ class AnsibleWrapper():
             dst_dir.mkdir(parents=True, exist_ok=True)
 
             for group in group_vars:
-                log.debug(f"Dumping group vars for [{group}]: "
-                          f"{group_vars[group]}")
+                log.debug(f"Dumping group vars for [{group}]: " f"{group_vars[group]}")
 
                 dst = Path(dst_dir, group + ".yml")
                 with open(dst, "w") as fp:
@@ -149,7 +160,7 @@ class AnsibleWrapper():
             with open(dst, "w") as fp:
                 yaml.dump(extravars, fp)
 
-    def _run(self, params, **kwargs):
+    def _run(self, params: Any, **kwargs: Any) -> Runner:
         """
         The actual entry point into the ansible_runner package.
 
@@ -176,7 +187,9 @@ class AnsibleWrapper():
 
         return runner
 
-    def get_inventory(self):
+    def get_inventory(
+        self,
+    ) -> Any:
         """
         Returns a YAML-formatted Ansible inventory populated from all sources.
 
@@ -207,10 +220,12 @@ class AnsibleWrapper():
         # we try to parse the returned inventory, so we'll just let the YAML
         # parser fail and hopefully it'll have more details for us.
         try:
-            inventory, _ = query_inventory(action="list",
-                                           inventories=[inventory_path],
-                                           response_format="yaml",
-                                           **params)
+            inventory, _ = query_inventory(
+                action="list",
+                inventories=[inventory_path],
+                response_format="yaml",
+                **params,
+            )
         except ansible_runner.exceptions.AnsibleRunnerException as ex:
             raise ExecutionError(f"ansible-runner failed: {ex}")
 
@@ -222,7 +237,9 @@ class AnsibleWrapper():
                 f"Got this from Ansible: {inventory}"
             )
 
-    def run_playbook(self, limit=None, verbosity=0):
+    def run_playbook(
+        self, limit: Optional[List[str]] = None, verbosity: int = 0
+    ) -> Runner:
         """
         :param limit: list of hosts to restrict the playbook execution to
         :param verbosity: verbosity of underlying ansible invocation
@@ -235,6 +252,6 @@ class AnsibleWrapper():
         if verbosity:
             params["verbosity"] = verbosity
         if limit:
-            params["limit"] = ','.join(limit)
+            params["limit"] = ",".join(limit)
 
         self._run(params)
