@@ -16,8 +16,12 @@ cli_args = [
     ["projects"],
     ["variables", "almalinux-9", "libvirt-go-module"],
     ["dockerfile", "almalinux-9", "libvirt-go-module"],
-    ["manifest", "-n", Path(__file__).parent.parent.joinpath("examples", "manifest.yml")],
-    ["container", "engines"]
+    [
+        "manifest",
+        "-n",
+        Path(__file__).parent.parent.joinpath("examples", "manifest.yml"),
+    ],
+    ["container", "engines"],
 ]
 
 
@@ -30,5 +34,7 @@ def test_commands(test_cli_args):
         # we're running the tests in a virtual env
         lcitool_path = Path(sys.prefix, "bin/lcitool")
 
-    subprocess.check_call([lcitool_path, '-d', Path(__file__).parent.joinpath("data")] + test_cli_args,
-                          stdout=subprocess.DEVNULL)
+    subprocess.check_call(
+        [lcitool_path, "-d", Path(__file__).parent.joinpath("data")] + test_cli_args,
+        stdout=subprocess.DEVNULL,
+    )
